"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AbortError = exports.GeneralError = exports.TimeoutError = exports.HTTPHeader = void 0;
const HTTPUtils_1 = require("./HTTPUtils");
const DEFAULT_TIMEOUT = 10000;
class HTTPHeader {
    constructor(initialHeaders) {
        if (typeof initialHeaders === 'undefined') {
            this.header = {};
        }
        else if (initialHeaders instanceof XMLHttpRequest) {
            this.header = HTTPHeader.fromXHR(initialHeaders);
        }
        else {
            this.header = HTTPHeader.fromMap(initialHeaders);
        }
    }
    static fromXHR(xhr) {
        let responseHeaders = xhr.getAllResponseHeaders();
        if (!responseHeaders) {
            return {};
        }
        let fieldsArray = responseHeaders.split(/\n/);
        var fieldsMap = {};
        fieldsArray.forEach((field) => {
            let firstColon = field.indexOf(':', 0);
            let name = field.substr(0, firstColon).trim();
            let value = field.substr(firstColon + 1).trim();
            fieldsMap[name.toLowerCase()] = value;
        });
        return fieldsMap;
    }
    static fromMap(header) {
        var fieldsMap = {};
        Object.keys(header).forEach((name) => {
            fieldsMap[name.toLowerCase()] = header[name];
        });
        return fieldsMap;
    }
    getHeader(fieldName) {
        return this.header[fieldName.toLowerCase()];
    }
    setHeader(fieldName, fieldValue) {
        this.header[fieldName.toLowerCase()] = fieldValue;
    }
    exportHeader(xhr) {
        Object.keys(this.header)
            .filter((key) => {
            if (this.getHeader(key) === undefined ||
                this.getHeader(key) === null) {
                return false;
            }
            return true;
        })
            .forEach((key) => {
            var stringValue = ((value) => {
                switch (typeof value) {
                    case 'string': return value;
                    case 'number': return String(value);
                    case 'boolean': return String(value);
                    default:
                        throw new Error('Invalid type for header value: ' + typeof value);
                }
            })(this.getHeader(key));
            xhr.setRequestHeader(key, stringValue);
        });
    }
    getContentType() {
        let value = this.header['content-type'];
        if (!value) {
            return null;
        }
        let values = value.split(';').map((x) => x.trim());
        if (values[1]) {
            return {
                mediaType: values[0],
                charset: values[1]
            };
        }
        else {
            return {
                mediaType: values[0]
            };
        }
    }
}
exports.HTTPHeader = HTTPHeader;
class TimeoutError extends Error {
    constructor(timeout, elapsed, message, xhr) {
        super(message);
        Object.setPrototypeOf(this, TimeoutError.prototype);
        this.name = 'TimeoutError';
        this.stack = new Error().stack;
        this.timeout = timeout;
        this.elapsed = elapsed;
        this.xhr = xhr;
    }
    toString() {
        return this.message;
    }
}
exports.TimeoutError = TimeoutError;
class GeneralError extends Error {
    constructor(message, xhr) {
        super(message);
        Object.setPrototypeOf(this, GeneralError.prototype);
        this.name = 'GeneralError';
        this.stack = new Error().stack;
        this.xhr = xhr;
    }
    toString() {
        return this.message;
    }
}
exports.GeneralError = GeneralError;
class AbortError extends Error {
    constructor(message, xhr) {
        super(message);
        Object.setPrototypeOf(this, AbortError.prototype);
        this.name = 'AbortError';
        this.stack = new Error().stack;
        this.xhr = xhr;
    }
    toString() {
        return this.message;
    }
}
exports.AbortError = AbortError;
class HTTPClient {
    constructor(options) {
        this.options = options;
    }
    request(options) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const startTime = new Date().getTime();
            const timeout = options.timeout || ((_a = this.options) === null || _a === void 0 ? void 0 : _a.timeout) || DEFAULT_TIMEOUT;
            return new Promise((resolve, reject) => {
                const xhr = new XMLHttpRequest();
                xhr.onload = () => {
                    switch (xhr.responseType) {
                        case 'text':
                            resolve({
                                status: xhr.status,
                                response: xhr.response,
                                responseType: xhr.responseType,
                                header: new HTTPHeader(xhr)
                            });
                            return;
                        default:
                            throw new Error(`Unsupported response type ${xhr.responseType}`);
                    }
                };
                xhr.ontimeout = () => {
                    var elapsed = (new Date().getTime()) - startTime;
                    reject(new TimeoutError(timeout, elapsed, 'Request timeout', xhr));
                };
                xhr.onerror = () => {
                    reject(new GeneralError('General request error ' + options.url, xhr));
                };
                xhr.onabort = () => {
                    reject(new AbortError('Request was aborted', xhr));
                };
                var url = options.url;
                if (options.query) {
                    url += '?' + new HTTPUtils_1.HTTPQuery(options.query).toString();
                }
                const rt = (options.responseType || 'text');
                xhr.responseType = rt;
                try {
                    xhr.open(options.method, url, true);
                }
                catch (ex) {
                    reject(new GeneralError('Error opening request ' + ex.name, xhr));
                    return;
                }
                xhr.timeout = timeout;
                xhr.withCredentials = options.withCredentials || false;
                try {
                    if (options.header) {
                        options.header.exportHeader(xhr);
                    }
                }
                catch (ex) {
                    reject(new GeneralError('Error applying header before send ' + ex.name, xhr));
                }
                try {
                    if (typeof options.data === 'string') {
                        xhr.send(options.data);
                        if (options.onCancel) {
                            options.onCancel(() => {
                                xhr.abort();
                            });
                        }
                    }
                    else if (options.data instanceof Array) {
                        xhr.send(new Uint8Array(options.data));
                    }
                    else if (typeof options.data === 'undefined') {
                        xhr.send();
                    }
                    else if (options.data === null) {
                        xhr.send();
                    }
                    else {
                        reject(new Error('Invalid type of data to send: ' + typeof options.data));
                    }
                }
                catch (ex) {
                    reject(new GeneralError('Error sending data in request', xhr));
                }
            });
        });
    }
}
exports.default = HTTPClient;
//# sourceMappingURL=HTTPClient.js.map